/*
 * CustomerBean
 */

import javax.ejb.CreateException;
import javax.ejb.DuplicateKeyException;
import javax.ejb.EntityBean;
import javax.ejb.EntityContext;
import javax.ejb.EJBException;
import javax.ejb.FinderException;
import javax.ejb.ObjectNotFoundException;
import javax.ejb.RemoveException;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import java.rmi.RemoteException;

import java.sql.Connection;
import javax.sql.DataSource;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.Statement;
import java.sql.SQLException;

public class CustomerBean implements EntityBean {

    // Variables especficas de EJB
    protected static final String CUSTOMER_DB = "java:comp/env/jdbc/CustomerDataSource";
    protected EntityContext entityContext;
    protected DataSource dataSource;


    // Variables de negocio (informacin de usuarios)
    protected CustomerTO information;


    // Sobreescribir los mtodos del interfaz EntityBean 
    public void setEntityContext(EntityContext ec) {
        
		entityContext = ec;

        try {
            InitialContext ic = new InitialContext();
            dataSource = (DataSource)ic.lookup(CUSTOMER_DB);
        } catch (Exception e) {
            e.printStackTrace();
            throw new EJBException(e);
        }
    }

    public void unsetEntityContext() {
       
		entityContext = null;
		dataSource = null;
    }

    // Mtodo ejbCreate que permite insertar un nuevo cliente en la base de datos
    public String ejbCreate(String i, String un, String add, String pn, String pass) 
        throws EJBException, DuplicateKeyException, CreateException {

        String ret = null;

		// Chequea duplicados
        if (dbFindCustomer(i) != null) {
            throw new DuplicateKeyException();
        }

        // Inserta un usuario.
        try {
            Connection c = dataSource.getConnection();
            PreparedStatement ps = 
                c.prepareStatement("insert into swc.users values (?, ?, ?, ?, ?)");
            ps.setString(1, i);
            ps.setString(2, un);
            ps.setString(3, add);
            ps.setString(4, pn);
    	    ps.setString(5, pass);
            ps.executeUpdate();
            ps.close();
            c.close();
            ret = i;
        }
        catch (SQLException e) {
            throw new EJBException(e);
        }        

        if (ret != null) {
            return ret;
        } else {
            throw new CreateException(i);
        }
    }


    public void ejbLoad() {
		try {
            Connection c = null;
            PreparedStatement ps = null;
            ResultSet rs = null;
	    
            c = dataSource.getConnection();
            ps = c.prepareStatement("select id, username, address, phonenumber, password"
                                    + " from swc.users"
                                    + " where id = ?");
            String id = (String) entityContext.getPrimaryKey();
            ps.setString(1, id);
            rs = ps.executeQuery();
            if (rs.next()) {
                information = new CustomerTO(rs.getString(1), rs.getString(2),
                                   rs.getString(3), rs.getString(4), rs.getString(5));
            }
            rs.close();
            ps.close();
            c.close();
        }
        catch (SQLException e) {
            throw new EJBException(e);
        }
    }

    // Busca un cliente en la base de datos
    public String ejbFindByPrimaryKey(String key) throws FinderException {
	
		String ret = dbFindCustomer(key);
	
		if (ret != null) {
			return ret;
		} else {
		    throw new ObjectNotFoundException(key);
		}
    }

    public void ejbPostCreate(String i, String un, String add, String pn, String pass) 
        throws EJBException, DuplicateKeyException, CreateException {}

    public void ejbActivate() {}

    public void ejbPassivate() {}

    public void ejbStore() {}

    public void ejbRemove() {}


    // Implementacin de los mtodos de negocio definidos en su
    // interfaz remota
    public CustomerTO getInformation() { 
		return information; 
    }

    // Mtodos auxiliares

	// Comprueba si existe un usuario dado su identificador
    String dbFindCustomer(String id) {
        try {
            Connection c = null;
            PreparedStatement ps = null;
            ResultSet rs = null;

            c = dataSource.getConnection();

            ps = c.prepareStatement("select id, username"
                                    + " from swc.users"
                                    + " where id = ?");

            ps.setString(1, id);
            rs = ps.executeQuery();

            String ret = rs.next() ? id : null;

            rs.close();
            ps.close();
            c.close();
	    
            return ret;
        } catch (SQLException e) {
            throw new EJBException(e);
        }
    }

}
